#ifndef RK_DNSPACK_ANNEALING_H
#define RK_DNSPACK_ANNEALING_H

#include <valarray>
#include <vector>

#include "SphereLabel.h"
#include "PackingStructure.h"
#include "MakeMdFile.h"


namespace rk
{
	namespace dnspack
	{
		class Annealing
		{
		public:
			Annealing(PackingStructure& packingStructure, const int maxStepNumber, const int neighborUpdateInterval, const double maxDisplacement);
			virtual ~Annealing();

		public:
			bool operator()(); // volumeɂȂꍇfalseԂ
			bool operator()(MakeMdFile& makeMdFile);

		private:
			bool setNeighborSpheres();
			void calculateGradient();
			void optimize();

			double potential(const double z) const noexcept;
			double force(const double z) const noexcept;

		private:
			PackingStructure& m_packingStructure;

			const int m_spheresNumber;
			const int m_maxStepNumber;
			const int m_neighborUpdateInterval;

			std::vector<std::pair<int, SphereLabel>> m_neighborSpheres{}; // baseIndex <= sphereIndex̏ꍇ̂
			std::valarray<double> m_gradientVector{};

			double m_maxDisplacement;
			const double m_pressure{ 0.1 };

		private:
			Annealing(const Annealing&) = delete;
			Annealing(Annealing&&) noexcept = delete;
			Annealing& operator=(const Annealing&) = delete;
			Annealing& operator=(Annealing&&) noexcept = delete;
		};
	}
}


inline bool rk::dnspack::Annealing::operator()()
{
	for (int rep = 0; rep < 20; ++rep)
	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < 10; ++step)
			{
				calculateGradient();
				optimize();
			}
		}

		else
		{
			return false;
		}
	}

	if (m_packingStructure.getBravaisLattice().getVolume() < 0)
	{
		return false;
	}

	int repNumber = m_maxStepNumber / m_neighborUpdateInterval;
	int repSurplus = m_maxStepNumber % m_neighborUpdateInterval;

	for (int rep = 0; rep < repNumber; ++rep)
	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < m_neighborUpdateInterval; ++step)
			{
				calculateGradient();
				optimize();
			}

			if (m_packingStructure.getBravaisLattice().getVolume() < 0)
			{
				return false;
			}
		}

		else
		{
			return false;
		}
	}

	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < repSurplus; ++step)
			{
				calculateGradient();
				optimize();
			}

			if (m_packingStructure.getBravaisLattice().getVolume() < 0)
			{
				return false;
			}
		}

		else
		{
			return false;
		}
	}

	return true;
}


inline bool rk::dnspack::Annealing::operator()(MakeMdFile& makeMdFile)
{
	makeMdFile.takePicture("Annealing");

	for (int rep = 0; rep < 20; ++rep)
	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < 10; ++step)
			{
				calculateGradient();
				optimize();
			}

			makeMdFile.takePicture();
		}

		else
		{
			return false;
		}
	}

	if (m_packingStructure.getBravaisLattice().getVolume() < 0)
	{
		return false;
	}

	int repNumber = m_maxStepNumber / m_neighborUpdateInterval;
	int repSurplus = m_maxStepNumber % m_neighborUpdateInterval;

	for (int rep = 0; rep < repNumber; ++rep)
	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < m_neighborUpdateInterval; ++step)
			{
				calculateGradient();
				optimize();
			}

			if (m_packingStructure.getBravaisLattice().getVolume() < 0)
			{
				return false;
			}

			makeMdFile.takePicture();
		}

		else
		{
			return false;
		}
	}

	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < repSurplus; ++step)
			{
				calculateGradient();
				optimize();
			}

			if (m_packingStructure.getBravaisLattice().getVolume() < 0)
			{
				return false;
			}
		}

		else
		{
			return false;
		}
	}

	makeMdFile.takePicture();
	return true;
}




inline double rk::dnspack::Annealing::potential(const double z) const noexcept
{
	if (z < 0)
	{
		return (-1.0 * z);
	}

	else
	{
		return (0);
	}
}

inline double rk::dnspack::Annealing::force(const double z) const noexcept
{
	if (z < 0)
	{
		return (-1.0);
	}

	else
	{
		return (0);
	}
}


#endif // !RK_DNSPACK_ANNEALING_H

