#ifndef RK_DNSPACK_ENVIRONMENTVARIABLES_H
#define RK_DNSPACK_ENVIRONMENTVARIABLES_H

#include <string>

#include "ExploreVariables.h"
#include "ReOptimizeVariables.h"


namespace rk
{
	namespace dnspack
	{
		class EnvironmentVariables
		{
		public:
			EnvironmentVariables();
			virtual ~EnvironmentVariables();

		public:
			static int getThreadNumber() noexcept;
			static const std::string& getOutputDirectoryName() noexcept;
			static void setThreadNumber(const int threadNumber);
			static void setOutputDirectoryName(const std::string& outputDirectoryName);

			static const ExploreVariables& getExploreVariables() noexcept;
			static void setExploreVariables(const ExploreVariables& exploreVariables);

			static const ReOptimizeVariables& getReOptimizeVariables() noexcept;
			static void setReOptimizeVariables(const ReOptimizeVariables& reOptimizeVariables);

		private:
			static int m_threadNumber;
			static std::string m_outputDirectoryName;

			static ExploreVariables m_exploreVariables;
			static ReOptimizeVariables m_reOptimizeVariables;

		private:
			EnvironmentVariables(const EnvironmentVariables& other) = delete;
			EnvironmentVariables(EnvironmentVariables&& other) noexcept = delete;
			EnvironmentVariables& operator=(const EnvironmentVariables& other) = delete;
			EnvironmentVariables& operator=(EnvironmentVariables&& other) noexcept = delete;
		};
	}
}


inline int rk::dnspack::EnvironmentVariables::getThreadNumber() noexcept
{
	return m_threadNumber;
}

inline void rk::dnspack::EnvironmentVariables::setThreadNumber(const int threadNumber)
{
	if (threadNumber < 1)
	{
		throw std::logic_error("threadNumber is less than 1.");
	}

	m_threadNumber = threadNumber;
}

inline const std::string& rk::dnspack::EnvironmentVariables::getOutputDirectoryName() noexcept
{
	return m_outputDirectoryName;
}

inline void rk::dnspack::EnvironmentVariables::setOutputDirectoryName(const std::string& outputDirectoryName)
{
	m_outputDirectoryName = outputDirectoryName;
}

inline const rk::dnspack::ExploreVariables& rk::dnspack::EnvironmentVariables::getExploreVariables() noexcept
{
	return m_exploreVariables;
}

inline void rk::dnspack::EnvironmentVariables::setExploreVariables(const ExploreVariables& exploreVariables)
{
	m_exploreVariables = exploreVariables;
}

inline const rk::dnspack::ReOptimizeVariables& rk::dnspack::EnvironmentVariables::getReOptimizeVariables() noexcept
{
	return m_reOptimizeVariables;
}

inline void rk::dnspack::EnvironmentVariables::setReOptimizeVariables(const ReOptimizeVariables& reOptimizeVariables)
{
	m_reOptimizeVariables = reOptimizeVariables;
}


#endif // !RK_DNSPACK_ENVIRONMENTVARIABLES_H
