#ifndef RK_DNSPACK_EXPLOREVARIABLES_H
#define RK_DNSPACK_EXPLOREVARIABLES_H

#include <stdexcept>


namespace rk
{
	namespace dnspack
	{
		class ExploreVariables
		{
		public:
			ExploreVariables() {}
			virtual ~ExploreVariables() {}

		public:
			ExploreVariables(const ExploreVariables&) = default;
			ExploreVariables(ExploreVariables&&) noexcept = default;
			ExploreVariables& operator=(const ExploreVariables&) = default;
			ExploreVariables& operator=(ExploreVariables&&) noexcept = default;

		public:

// ********************************************************************************************************************************************************************
			// Optimization Parameters

			double getAnnealingMaxDisplacementUpper() const noexcept;
			double getAnnealingMaxDisplacementLower() const noexcept;
			int getAnnealingIterationNumber() const noexcept;
			int getAnnealingNeighborUpdateInterval() const noexcept;
			void setAnnealingMaxDisplacement(const double& annealingMaxDisplacementUpper, const double& annealingMaxDisplacementLower);
			void setAnnealingIterationNumber(const int& annealingIteratorNumber);
			void setAnnealingNeighborUpdateInterval(const int& annealingNeighborUpdateInterval);

			double getInitialPackingOptimizeDecreaseFactor() const noexcept;
			int getInitialPackingOptimizeIterationNumber() const noexcept;
			int getInitialPackingOptimizeNeighborUpdateInterval() const noexcept;
			void setInitialPackingOptimizeDecreaseFactor(const double& initialPackingOptimizeDecreaseFactor);
			void setInitialPackingOptimizeIterationNumber(const int& initialPackingOptimizeIteratorNumber);
			void setInitialPackingOptimizeNeighborUpdateInterval(const int& initialPackingOptimizeNeighborUpdateInterval);

			double getPackingOptimizeMaxDisplacementUpper() const noexcept;
			double getPackingOptimizeMaxDisplacementLower() const noexcept;
			double getPackingOptimizeDecreaseFactor() const noexcept;
			int getPackingOptimizeIterationNumber() const noexcept;
			int getPackingOptimizeNeighborUpdateInterval() const noexcept;
			void setPackingOptimizeMaxDisplacement(const double& packingOptimizeMaxDisplacementUpper, const double& packingOptimizeMaxDisplacementLower);
			void setPackingOptimizeDecreaseFactor(const double& packingOptimizeDecreaseFactor);
			void setPackingOptimizeIterationNumber(const int& packingOptimizeIteratorNumber);
			void setPackingOptimizeNeighborUpdateInterval(const int& packingOptimizeNeighborUpdateInterval);

			// Optimization Parameters
// ********************************************************************************************************************************************************************

// ********************************************************************************************************************************************************************
			// Conditions

			double getUniformGenerationRatio() const noexcept;
			void setUniformGenerationRatio(const double uniformGenerationRatio);

			int getFullOptimizeStructureNumber() const noexcept;
			int getMaxTrialNumber() const noexcept;
			int getThresholdTrialNumber() const noexcept;
			void setFullOptimizeStructureNumber(const int fullOptimizeStructureNumber);
			void setMaxTrialNumber(const int maxTrialNumber);
			void setThresholdTrialNumber(const int thresholdTrialNumber);

			double getOutputMinimumPackingFraction() const noexcept;
			double getOutputMinimumZ() const noexcept;
			bool outputMdFile() const noexcept;
			void setOutputMinimumPackingFraction(const double outputMinimumPackingFraction);
			void setOutputMinimumZ(const double outputMinimumZ);
			void setOutputMdFile(const bool outputMdFile);

			double getPackingFractionsErrorMesh() const noexcept;
			double getInitialPackingOptimizePackingFractionsMesh() const noexcept;
			double getPackingOptimizePackingFractionsMesh() const noexcept;
			void setPackingFractionsErrorMesh(const double packingFractionsErrorMesh);
			void setInitialPackingOptimizePackingFractionsMesh(const double smallAnnealingPackingFractionsMesh);
			void setPackingOptimizePackingFractionsMesh(const double packingOptimizePackingFractionsMesh);

			// Conditions
// ********************************************************************************************************************************************************************

		private:

// ********************************************************************************************************************************************************************
			// Optimization Parameters

			double m_annealingMaxDisplacementUpper{ 0.0 };
			double m_annealingMaxDisplacementLower{ 0.0 };
			int m_annealingIterationNumber{ 0 };
			int m_annealingNeighborUpdateInterval{ 0 };

			double m_initialPackingOptimizeDecreaseFactor{ 0.0 };
			int m_initialPackingOptimizeIterationNumber{ 0 };
			int m_initialPackingOptimizeNeighborUpdateInterval{ 0 };

			double m_packingOptimizeMaxDisplacementUpper{ 0.0 };
			double m_packingOptimizeMaxDisplacementLower{ 0.0 };
			double m_packingOptimizeDecreaseFactor{ 0.0 };
			int m_packingOptimizeIterationNumber{ 0 };
			int m_packingOptimizeNeighborUpdateInterval{ 0 };

			// Optimization Parameters
// ********************************************************************************************************************************************************************

// ********************************************************************************************************************************************************************
			// Conditions

			double m_uniformGenerationRatio{ 0.0 };

			int m_fullOptimizeStructureNumber{ 0 };
			int m_maxTrialNumber{ 0 };
			int m_thresholdTrialNumber{ 0 };

			double m_outputMinimumPackingFraction{ 0.0 };
			double m_outputMinimumZ{ 0.0 };
			bool m_outputMdFile{ false };

			double m_packingFractionsErrorMesh{ 0.0 };
			double m_initialPackingOptimizePackingFractionsMesh{ 0.0 };
			double m_packingOptimizePackingFractionsMesh{ 0.0 };

			// Conditions
// ********************************************************************************************************************************************************************

		};
	}
}


// ********************************************************************************************************************************************************************
	// Annealing

inline double rk::dnspack::ExploreVariables::getAnnealingMaxDisplacementUpper() const noexcept
{
	return m_annealingMaxDisplacementUpper;
}

inline double rk::dnspack::ExploreVariables::getAnnealingMaxDisplacementLower() const noexcept
{
	return m_annealingMaxDisplacementLower;
}

inline int rk::dnspack::ExploreVariables::getAnnealingIterationNumber() const noexcept
{
	return m_annealingIterationNumber;
}

inline int rk::dnspack::ExploreVariables::getAnnealingNeighborUpdateInterval() const noexcept
{
	return m_annealingNeighborUpdateInterval;
}

inline void rk::dnspack::ExploreVariables::setAnnealingMaxDisplacement(const double& annealingMaxDisplacementUpper, const double& annealingMaxDisplacementLower)
{
	if (annealingMaxDisplacementUpper < annealingMaxDisplacementLower)
	{
		throw std::logic_error("annealingMaxDisplacementUpper is less than annealingMaxDisplacementLower.");
	}

	if (0.2 < annealingMaxDisplacementUpper)
	{
		throw std::logic_error("annealingMaxDisplacementUpper is larger than 0.2.");
	}

	m_annealingMaxDisplacementUpper = annealingMaxDisplacementUpper;
	m_annealingMaxDisplacementLower = annealingMaxDisplacementLower;
}

inline void rk::dnspack::ExploreVariables::setAnnealingIterationNumber(const int& annealingIteratorNumber)
{
	if (annealingIteratorNumber < 0)
	{
		throw std::logic_error("annealingIteratorNumber is less than zero.");
	}

	m_annealingIterationNumber = annealingIteratorNumber;
}

inline void rk::dnspack::ExploreVariables::setAnnealingNeighborUpdateInterval(const int& annealingNeighborUpdateInterval)
{
	if (annealingNeighborUpdateInterval <= 0)
	{
		throw std::logic_error("annealingNeighborUpdateInterval have to be larger than zero.");
	}

	m_annealingNeighborUpdateInterval = annealingNeighborUpdateInterval;
}

	// Annealing
// ********************************************************************************************************************************************************************


// ********************************************************************************************************************************************************************
	// Initial Packing Optimize

inline double rk::dnspack::ExploreVariables::getInitialPackingOptimizeDecreaseFactor() const noexcept
{
	return m_initialPackingOptimizeDecreaseFactor;
}

inline int rk::dnspack::ExploreVariables::getInitialPackingOptimizeIterationNumber() const noexcept
{
	return m_initialPackingOptimizeIterationNumber;
}

inline int rk::dnspack::ExploreVariables::getInitialPackingOptimizeNeighborUpdateInterval() const noexcept
{
	return m_initialPackingOptimizeNeighborUpdateInterval;
}

inline void rk::dnspack::ExploreVariables::setInitialPackingOptimizeDecreaseFactor(const double& initialPackingOptimizeDecreaseFactor)
{
	if (initialPackingOptimizeDecreaseFactor < 0.0)
	{
		throw std::logic_error("packingOptimizeDecreaseFactor is less than zero.");
	}

	m_initialPackingOptimizeDecreaseFactor = initialPackingOptimizeDecreaseFactor;
}

inline void rk::dnspack::ExploreVariables::setInitialPackingOptimizeIterationNumber(const int& initialPackingOptimizeIteratorNumber)
{
	if (initialPackingOptimizeIteratorNumber < 0)
	{
		throw std::logic_error("packingOptimizeIteratorNumber is less than zero.");
	}

	m_initialPackingOptimizeIterationNumber = initialPackingOptimizeIteratorNumber;
}

inline void rk::dnspack::ExploreVariables::setInitialPackingOptimizeNeighborUpdateInterval(const int& initialPackingOptimizeNeighborUpdateInterval)
{
	if (initialPackingOptimizeNeighborUpdateInterval <= 0)
	{
		throw std::logic_error("packingOptimizeNeighborUpdateInterval have to be larger than zero.");
	}

	m_initialPackingOptimizeNeighborUpdateInterval = initialPackingOptimizeNeighborUpdateInterval;
}

	// Initial Packing Optimize
// ********************************************************************************************************************************************************************


// ********************************************************************************************************************************************************************
	// Packing Optimize

inline double rk::dnspack::ExploreVariables::getPackingOptimizeMaxDisplacementUpper() const noexcept
{
	return m_packingOptimizeMaxDisplacementUpper;
}

inline double rk::dnspack::ExploreVariables::getPackingOptimizeMaxDisplacementLower() const noexcept
{
	return m_packingOptimizeMaxDisplacementLower;
}

inline double rk::dnspack::ExploreVariables::getPackingOptimizeDecreaseFactor() const noexcept
{
	return m_packingOptimizeDecreaseFactor;
}

inline int rk::dnspack::ExploreVariables::getPackingOptimizeIterationNumber() const noexcept
{
	return m_packingOptimizeIterationNumber;
}

inline int rk::dnspack::ExploreVariables::getPackingOptimizeNeighborUpdateInterval() const noexcept
{
	return m_packingOptimizeNeighborUpdateInterval;
}

inline void rk::dnspack::ExploreVariables::setPackingOptimizeMaxDisplacement(const double& packingOptimizeMaxDisplacementUpper, const double& packingOptimizeMaxDisplacementLower)
{
	if (packingOptimizeMaxDisplacementUpper < packingOptimizeMaxDisplacementLower)
	{
		throw std::runtime_error("packingOptimizeMaxDisplacementUpper is less than packingOptimizeMaxDisplacementLower.");
	}

	if (0.2 < packingOptimizeMaxDisplacementUpper)
	{
		throw std::runtime_error("m_packingOptimizeMaxDisplacementUpper is larger than 0.2.");
	}

	m_packingOptimizeMaxDisplacementUpper = packingOptimizeMaxDisplacementUpper;
	m_packingOptimizeMaxDisplacementLower = packingOptimizeMaxDisplacementLower;
}

inline void rk::dnspack::ExploreVariables::setPackingOptimizeDecreaseFactor(const double& packingOptimizeDecreaseFactor)
{
	if (packingOptimizeDecreaseFactor < 0.0)
	{
		throw std::logic_error("packingOptimizeDecreaseFactor is less than zero.");
	}

	m_packingOptimizeDecreaseFactor = packingOptimizeDecreaseFactor;
}

inline void rk::dnspack::ExploreVariables::setPackingOptimizeIterationNumber(const int& packingOptimizeIteratorNumber)
{
	if (packingOptimizeIteratorNumber < 0)
	{
		throw std::logic_error("packingOptimizeIteratorNumber is less than zero.");
	}

	m_packingOptimizeIterationNumber = packingOptimizeIteratorNumber;
}

inline void rk::dnspack::ExploreVariables::setPackingOptimizeNeighborUpdateInterval(const int& packingOptimizeNeighborUpdateInterval)
{
	if (packingOptimizeNeighborUpdateInterval <= 0)
	{
		throw std::logic_error("packingOptimizeNeighborUpdateInterval have to be larger than zero.");
	}

	m_packingOptimizeNeighborUpdateInterval = packingOptimizeNeighborUpdateInterval;
}

	// Packing Optimize
// ********************************************************************************************************************************************************************


// ********************************************************************************************************************************************************************
	// Conditions

inline double rk::dnspack::ExploreVariables::getUniformGenerationRatio() const noexcept
{
	return m_uniformGenerationRatio;
}

inline void rk::dnspack::ExploreVariables::setUniformGenerationRatio(const double uniformGenerationRatio)
{
	if (1.0 < uniformGenerationRatio)
	{
		throw std::logic_error("fullOptimizeStructureNumber is more than one.");
	}

	else if (uniformGenerationRatio < 0.0)
	{
		throw std::logic_error("fullOptimizeStructureNumber is more than zero.");
	}

	else
	{
		m_uniformGenerationRatio = uniformGenerationRatio;
	}
}

inline int rk::dnspack::ExploreVariables::getFullOptimizeStructureNumber() const noexcept
{
	return m_fullOptimizeStructureNumber;
}

inline int rk::dnspack::ExploreVariables::getMaxTrialNumber() const noexcept
{
	return m_maxTrialNumber;
}

inline int rk::dnspack::ExploreVariables::getThresholdTrialNumber() const noexcept
{
	return m_thresholdTrialNumber;
}

inline void rk::dnspack::ExploreVariables::setFullOptimizeStructureNumber(const int fullOptimizeStructureNumber)
{
	if (fullOptimizeStructureNumber < 1)
	{
		throw std::logic_error("fullOptimizeStructureNumber is less than one.");
	}

	m_fullOptimizeStructureNumber = fullOptimizeStructureNumber;
}

inline void rk::dnspack::ExploreVariables::setMaxTrialNumber(const int maxTrialNumber)
{
	if (maxTrialNumber < 1)
	{
		throw std::logic_error("maxTrialNumber is less than one.");
	}

	m_maxTrialNumber = maxTrialNumber;
}

inline void rk::dnspack::ExploreVariables::setThresholdTrialNumber(const int thresholdTrialNumber)
{
	if (thresholdTrialNumber < 1)
	{
		throw std::logic_error("thresholdTrialNumber is less than one.");
	}

	m_thresholdTrialNumber = thresholdTrialNumber;
}


inline double rk::dnspack::ExploreVariables::getOutputMinimumPackingFraction() const noexcept
{
	return m_outputMinimumPackingFraction;
}

inline double rk::dnspack::ExploreVariables::getOutputMinimumZ() const noexcept
{
	return m_outputMinimumZ;
}

inline bool rk::dnspack::ExploreVariables::outputMdFile() const noexcept
{
	return m_outputMdFile;
}

inline void rk::dnspack::ExploreVariables::setOutputMinimumPackingFraction(const double outputMinimumPackingFraction)
{
	if (outputMinimumPackingFraction < 0.0)
	{
		throw std::logic_error("outputMinimumPackingFraction is less than zero.");
	}

	m_outputMinimumPackingFraction = outputMinimumPackingFraction;
}

inline void rk::dnspack::ExploreVariables::setOutputMinimumZ(const double outputMinimumZ)
{
	if (0.0 <= outputMinimumZ)
	{
		throw std::logic_error("outputMinimumZ is more than or equal to zero.");
	}

	m_outputMinimumZ = outputMinimumZ;
}

inline void rk::dnspack::ExploreVariables::setOutputMdFile(const bool outputMdFile)
{
	m_outputMdFile = outputMdFile;
}


inline double rk::dnspack::ExploreVariables::getPackingFractionsErrorMesh() const noexcept
{
	return m_packingFractionsErrorMesh;
}

inline double rk::dnspack::ExploreVariables::getInitialPackingOptimizePackingFractionsMesh() const noexcept
{
	return m_initialPackingOptimizePackingFractionsMesh;
}

inline double rk::dnspack::ExploreVariables::getPackingOptimizePackingFractionsMesh() const noexcept
{
	return m_packingOptimizePackingFractionsMesh;
}

inline void rk::dnspack::ExploreVariables::setPackingFractionsErrorMesh(const double packingFractionsErrorMesh)
{
	if (packingFractionsErrorMesh < 0.0)
	{
		throw std::logic_error("packingFractionsErrorMesh is less than zero.");
	}

	m_packingFractionsErrorMesh = packingFractionsErrorMesh;
}

inline void rk::dnspack::ExploreVariables::setInitialPackingOptimizePackingFractionsMesh(const double smallAnnealingPackingFractionsMesh)
{
	if (smallAnnealingPackingFractionsMesh < 0.0)
	{
		throw std::logic_error("packingFractionsErrorMesh is less than zero.");
	}

	m_initialPackingOptimizePackingFractionsMesh = smallAnnealingPackingFractionsMesh;
}

inline void rk::dnspack::ExploreVariables::setPackingOptimizePackingFractionsMesh(const double packingOptimizePackingFractionsMesh)
{
	if (packingOptimizePackingFractionsMesh < 0.0)
	{
		throw std::logic_error("packingOptimizePackingFractionsMesh is less than zero.");
	}

	m_packingOptimizePackingFractionsMesh = packingOptimizePackingFractionsMesh;
}

	// Conditions
// ********************************************************************************************************************************************************************


#endif // !RK_DNSPACK_EXPLOREVARIABLES_H

