#ifndef RK_DNSPACK_FRACTIONALCOORDINATE_H
#define RK_DNSPACK_FRACTIONALCOORDINATE_H

#include <stdexcept>

namespace rk
{
	namespace dnspack
	{
		class FractionalCoordinate
		{
		public:
			double a;
			double b;
			double c;

		public:
			FractionalCoordinate() noexcept;
			FractionalCoordinate(const double x, const double y, const double z) noexcept;
			virtual ~FractionalCoordinate();

		public:
			FractionalCoordinate(const FractionalCoordinate& other) noexcept;
			FractionalCoordinate(FractionalCoordinate&& other) noexcept;
			FractionalCoordinate& operator=(const FractionalCoordinate& other) noexcept;
			FractionalCoordinate& operator=(FractionalCoordinate&& other) noexcept;

			bool operator==(const FractionalCoordinate& other) const noexcept;
			bool operator!=(const FractionalCoordinate& other) const noexcept;

			FractionalCoordinate& operator+=(const FractionalCoordinate& other) noexcept;
			FractionalCoordinate& operator-=(const FractionalCoordinate& other) noexcept;
			FractionalCoordinate& scaling(const double num) noexcept;

			double operator[](const int direction) const;
		};
	}
}


inline rk::dnspack::FractionalCoordinate& rk::dnspack::FractionalCoordinate::operator=(const rk::dnspack::FractionalCoordinate& other) noexcept
{
	a = other.a;
	b = other.b;
	c = other.c;

	return *this;
}

inline rk::dnspack::FractionalCoordinate& rk::dnspack::FractionalCoordinate::operator=(rk::dnspack::FractionalCoordinate&& other) noexcept
{
	a = other.a;
	b = other.b;
	c = other.c;

	return *this;
}

inline bool rk::dnspack::FractionalCoordinate::operator==(const rk::dnspack::FractionalCoordinate& other) const noexcept
{
	return (a == other.a && b == other.b && c == other.c);
}

inline bool rk::dnspack::FractionalCoordinate::operator!=(const rk::dnspack::FractionalCoordinate& other) const noexcept
{
	return !(*this == other);
}

inline rk::dnspack::FractionalCoordinate& rk::dnspack::FractionalCoordinate::operator+=(const rk::dnspack::FractionalCoordinate& other) noexcept
{
	a += other.a;
	b += other.b;
	c += other.c;

	return *this;
}

inline rk::dnspack::FractionalCoordinate& rk::dnspack::FractionalCoordinate::operator-=(const rk::dnspack::FractionalCoordinate& other) noexcept
{
	a -= other.a;
	b -= other.b;
	c -= other.c;

	return *this;
}

inline rk::dnspack::FractionalCoordinate& rk::dnspack::FractionalCoordinate::scaling(const double num) noexcept
{
	a *= num;
	b *= num;
	c *= num;

	return *this;
}

inline double rk::dnspack::FractionalCoordinate::operator[](const int direction) const
{
	switch (direction)
	{
	case 0:
		return a;
		break;

	case 1:
		return b;
		break;

	case 2:
		return c;
		break;

	default:
		throw std::invalid_argument("FractionalCoordinate::operator[](const int direction) : direction is unsuitable.");
	}
}


namespace rk
{
	namespace dnspack
	{
		inline FractionalCoordinate operator+(FractionalCoordinate coordinate, const FractionalCoordinate& coordinate2)
		{
			coordinate.a += coordinate2.a;
			coordinate.b += coordinate2.b;
			coordinate.c += coordinate2.c;

			return coordinate;
		}

		inline FractionalCoordinate operator-(FractionalCoordinate coordinate, const FractionalCoordinate& coordinate2)
		{
			coordinate.a -= coordinate2.a;
			coordinate.b -= coordinate2.b;
			coordinate.c -= coordinate2.c;

			return coordinate;
		}

		inline FractionalCoordinate scaling(const double num, FractionalCoordinate coordinate)
		{
			coordinate.a *= num;
			coordinate.b *= num;
			coordinate.c *= num;

			return coordinate;
		}
	}
}


#endif // !RK_DNSPACK_FRACTIONALCOORDINATE_H
