#ifndef RK_DNSPACK_MAKEDATFILE_H
#define RK_DNSPACK_MAKEDATFILE_H

#include <string>
#include <fstream>
#include <mutex>

#include "PackingStructure.h"
#include "SAMLAIReadme.h"
#include "Clock.h"

namespace rk
{
	namespace dnspack
	{
		class MakeDatFile
		{
		public:
			MakeDatFile();
			virtual ~MakeDatFile();

		public:
			void operator()(const std::string& fileName, const PackingStructure& packingStructure);
			void operator()(const std::string& fileName, const PackingStructure& packingStructure, const std::string& information);

		private:
			void writePackingInfo(const std::string& fileName, const PackingStructure& packingStructure);

		private:
			static std::mutex m_datMutex;

		private:
			std::string m_outputString;

		private:
			MakeDatFile(const MakeDatFile& other) = delete;
			MakeDatFile(MakeDatFile&& other) noexcept = delete;
			MakeDatFile& operator=(const MakeDatFile& other) = delete;
			MakeDatFile& operator=(MakeDatFile&& other) noexcept = delete;
		};
	}
}


inline void rk::dnspack::MakeDatFile::operator()(const std::string& fileName, const rk::dnspack::PackingStructure& packingStructure)
{
	m_outputString += "*********************************************************************************************************\n\n";
	m_outputString += "This explore of the densest sphere packings was performed by SAMLAIsp " + SAMLAIVersion + "\n";
	m_outputString += Clock::getNowTime() + "\n\n";
	m_outputString += "*********************************************************************************************************\n\n\n";

	writePackingInfo(fileName, packingStructure);

	std::lock_guard<std::mutex> lock{ m_datMutex };

	std::ofstream outputFileStream;
	outputFileStream.open(fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		m_outputString.clear();
		throw std::invalid_argument("couldn't open dat file.");
	}

	outputFileStream << m_outputString;
	outputFileStream.close();

	m_outputString.clear();
}

inline void rk::dnspack::MakeDatFile::operator()(const std::string& fileName, const rk::dnspack::PackingStructure& packingStructure, const std::string& information)
{
	m_outputString += "**************************************************************************************************************\n\n";
	m_outputString += "This explore of the densest sphere packings was performed by SAMLAIsp " + SAMLAIVersion + "\n\n";
	m_outputString += Clock::getNowTime() + "\n\n";
	m_outputString += "**************************************************************************************************************\n\n\n";

	writePackingInfo(fileName, packingStructure);

	m_outputString += information;

	std::lock_guard<std::mutex> lock{ m_datMutex };

	std::ofstream outputFileStream;
	outputFileStream.open(fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		m_outputString.clear();
		throw std::invalid_argument("couldn't open dat file.");
	}

	outputFileStream << m_outputString;
	outputFileStream.close();

	m_outputString.clear();
}


#endif // !RK_DNSPACK_MAKEDATFILE_H
