#ifndef RK_DNSPACK_MAKEMDFILE_H
#define RK_DNSPACK_MAKEMDFILE_H

#include <string>
#include <fstream>
#include <map>
#include <mutex>

#include "PackingStructure.h"
#include "SAMLAIReadme.h"


namespace rk
{
	namespace dnspack
	{
		class MakeMdFile
		{
		public:
			explicit MakeMdFile(PackingStructure& packingStructure);
			virtual ~MakeMdFile();

		public:
			void operator()(const std::string& fileName) const;

			void takePicture();
			void takePicture(const std::string& message);

		private:
			PackingStructure& m_packingStructure;

			const int m_spheresNumber;
			std::string m_moveRecord;

			int m_stepNumber;
			std::map<double, std::string> m_radiiAndName;

		private:
			static std::mutex m_mdMutex;

		private:
			MakeMdFile(const MakeMdFile& other) = delete;
			MakeMdFile(MakeMdFile&& other) noexcept = delete;
			MakeMdFile& operator=(const MakeMdFile& other) = delete;
			MakeMdFile& operator=(MakeMdFile&& other) noexcept = delete;
		};
	}
}


inline void rk::dnspack::MakeMdFile::operator()(const std::string& fileName) const
{
	std::lock_guard<std::mutex> lock{ m_mdMutex };

	std::ofstream outputFileStream;
	outputFileStream.open(fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open md file.");
	}

	outputFileStream << m_moveRecord;
	outputFileStream.close();
}


#endif // !RK_DNSPACK_MAKEMDFILE_H
