#ifndef RK_DNSPACK_REOPTIMIZEVARIABLES_H
#define RK_DNSPACK_REOPTIMIZEVARIABLES_H

#include <stdexcept>


namespace rk
{
	namespace dnspack
	{
		class ReOptimizeVariables
		{
		public:
			ReOptimizeVariables() {}
			virtual ~ReOptimizeVariables() {}

		public:
			ReOptimizeVariables(const ReOptimizeVariables&) = default;
			ReOptimizeVariables(ReOptimizeVariables&&) noexcept = default;
			ReOptimizeVariables& operator=(const ReOptimizeVariables&) = default;
			ReOptimizeVariables& operator=(ReOptimizeVariables&&) noexcept = default;

		public:

// ********************************************************************************************************************************************************************
			// ReOptimization Parameters

			double getMaxFluctuation() const noexcept;
			void setMaxFluctuation(const double& maxFluctuation);

			double getInitialAnnealingMaxDisplacementUpper() const noexcept;
			double getInitialAnnealingMaxDisplacementLower() const noexcept;
			int getInitialAnnealingIterationNumber() const noexcept;
			int getInitialAnnealingNeighborUpdateInterval() const noexcept;
			void setInitialAnnealingMaxDisplacement(const double& initialAnnealingMaxDisplacementUpper, const double& initialAnnealingMaxDisplacementLower);
			void setInitialAnnealingIterationNumber(const int& initialAnnealingIteratorNumber);
			void setInitialAnnealingNeighborUpdateInterval(const int& initialAnnealingNeighborUpdateInterval);

			double getDensestPackingOptimizeMaxDisplacementUpper() const noexcept;
			double getDensestPackingOptimizeMaxDisplacementLower() const noexcept;
			double getDensestPackingOptimizeDecreaseFactor() const noexcept;
			int getDensestPackingOptimizeIterationNumber() const noexcept;
			int getDensestPackingOptimizeNeighborUpdateInterval() const noexcept;
			void setDensestPackingOptimizeMaxDisplacement(const double& densestPackingOptimizeMaxDisplacementUpper, const double& densestPackingOptimizeMaxDisplacementLower);
			void setDensestPackingOptimizeDecreaseFactor(const double& densestPackingOptimizeDecreaseFactor);
			void setDensestPackingOptimizeIterationNumber(const int& densestPackingOptimizeIteratorNumber);
			void setDensestPackingOptimizeNeighborUpdateInterval(const int& densestPackingOptimizeNeighborUpdateInterval);

			// ReOptimization Parameters
// ********************************************************************************************************************************************************************

// ********************************************************************************************************************************************************************
			// Conditions

			int getReOptimizeStructureNumber() const noexcept;
			int getReOptimizeRepetitionNumber() const noexcept;
			double getOutputMinimumZ() const noexcept;
			void setReOptimizeStructureNumber(const int& reOptimizeStructureNumber);
			void setReOptimizeRepetitionNumber(const int& reOptimizeRepetitionNumber);
			void setOutputMinimumZ(const double& outputMinimumZ);

			// Conditions
// ********************************************************************************************************************************************************************

		private:

// ********************************************************************************************************************************************************************
			// ReOptimization Parameters

			double m_maxFluctuation{ 0.0 };

			double m_initialAnnealingMaxDisplacementUpper{ 0.0 };
			double m_initialAnnealingMaxDisplacementLower{ 0.0 };
			int m_initialAnnealingIterationNumber{ 0 };
			int m_initialAnnealingNeighborUpdateInterval{ 0 };

			double m_densestPackingOptimizeMaxDisplacementUpper{ 0.0 };
			double m_densestPackingOptimizeMaxDisplacementLower{ 0.0 };
			double m_densestPackingOptimizeDecreaseFactor{ 0.0 };
			int m_densestPackingOptimizeIterationNumber{ 0 };
			int m_densestPackingOptimizeNeighborUpdateInterval{ 0 };

			// ReOptimization Parameters
// ********************************************************************************************************************************************************************

// ********************************************************************************************************************************************************************
			// Conditions

			int m_reOptimizeStructureNumber{ 0 };
			int m_reOptimizeRepetitionNumber{ 0 };
			double m_outputMinimumZ{ 0.0 };

			// Conditions
// ********************************************************************************************************************************************************************

		};
	}
}


// ********************************************************************************************************************************************************************
	// ReOptimization Parameters

inline double rk::dnspack::ReOptimizeVariables::getMaxFluctuation() const noexcept
{
	return m_maxFluctuation;
}

inline void rk::dnspack::ReOptimizeVariables::setMaxFluctuation(const double& maxFluctuation)
{
	if (maxFluctuation < 0)
	{
		throw std::logic_error("initialAnnealingIteratorNumber is less than zero.");
	}

	m_maxFluctuation = maxFluctuation;
}


inline double rk::dnspack::ReOptimizeVariables::getInitialAnnealingMaxDisplacementUpper() const noexcept
{
	return m_initialAnnealingMaxDisplacementUpper;
}

inline double rk::dnspack::ReOptimizeVariables::getInitialAnnealingMaxDisplacementLower() const noexcept
{
	return m_initialAnnealingMaxDisplacementLower;
}

inline int rk::dnspack::ReOptimizeVariables::getInitialAnnealingIterationNumber() const noexcept
{
	return m_initialAnnealingIterationNumber;
}

inline int rk::dnspack::ReOptimizeVariables::getInitialAnnealingNeighborUpdateInterval() const noexcept
{
	return m_initialAnnealingNeighborUpdateInterval;
}

inline void rk::dnspack::ReOptimizeVariables::setInitialAnnealingMaxDisplacement(const double& initialAnnealingMaxDisplacementUpper, const double& initialAnnealingMaxDisplacementLower)
{
	if (initialAnnealingMaxDisplacementUpper < initialAnnealingMaxDisplacementLower)
	{
		throw std::logic_error("initialAnnealingMaxDisplacementUpper is less than annealingMaxDisplacementLower.");
	}

	if (0.2 < initialAnnealingMaxDisplacementUpper)
	{
		throw std::logic_error("initialAnnealingMaxDisplacementUpper is larger than 0.2.");
	}

	m_initialAnnealingMaxDisplacementUpper = initialAnnealingMaxDisplacementUpper;
	m_initialAnnealingMaxDisplacementLower = initialAnnealingMaxDisplacementLower;
}

inline void rk::dnspack::ReOptimizeVariables::setInitialAnnealingIterationNumber(const int& initialAnnealingIteratorNumber)
{
	if (initialAnnealingIteratorNumber < 0)
	{
		throw std::logic_error("initialAnnealingIteratorNumber is less than zero.");
	}

	m_initialAnnealingIterationNumber = initialAnnealingIteratorNumber;
}

inline void rk::dnspack::ReOptimizeVariables::setInitialAnnealingNeighborUpdateInterval(const int& initialAnnealingNeighborUpdateInterval)
{
	if (initialAnnealingNeighborUpdateInterval <= 0)
	{
		throw std::logic_error("initialAnnealingNeighborUpdateInterval have to be larger than zero.");
	}

	m_initialAnnealingNeighborUpdateInterval = initialAnnealingNeighborUpdateInterval;
}


inline double rk::dnspack::ReOptimizeVariables::getDensestPackingOptimizeMaxDisplacementUpper() const noexcept
{
	return m_densestPackingOptimizeMaxDisplacementUpper;
}

inline double rk::dnspack::ReOptimizeVariables::getDensestPackingOptimizeMaxDisplacementLower() const noexcept
{
	return m_densestPackingOptimizeMaxDisplacementLower;
}

inline double rk::dnspack::ReOptimizeVariables::getDensestPackingOptimizeDecreaseFactor() const noexcept
{
	return m_densestPackingOptimizeDecreaseFactor;
}

inline int rk::dnspack::ReOptimizeVariables::getDensestPackingOptimizeIterationNumber() const noexcept
{
	return m_densestPackingOptimizeIterationNumber;
}

inline int rk::dnspack::ReOptimizeVariables::getDensestPackingOptimizeNeighborUpdateInterval() const noexcept
{
	return m_densestPackingOptimizeNeighborUpdateInterval;
}

inline void rk::dnspack::ReOptimizeVariables::setDensestPackingOptimizeMaxDisplacement(const double& densestPackingOptimizeMaxDisplacementUpper, const double& densestPackingOptimizeMaxDisplacementLower)
{
	if (densestPackingOptimizeMaxDisplacementUpper < densestPackingOptimizeMaxDisplacementLower)
	{
		throw std::runtime_error("DensestPackingOptimize.Maximum.Displacement.Upper is less than DensestPackingOptimize.Maximum.Displacement.Lower.");
	}

	if (0.2 < densestPackingOptimizeMaxDisplacementUpper)
	{
		throw std::runtime_error("PackingOptimize.Maximum.Displacement.Upper is larger than 0.2.");
	}

	m_densestPackingOptimizeMaxDisplacementUpper = densestPackingOptimizeMaxDisplacementUpper;
	m_densestPackingOptimizeMaxDisplacementLower = densestPackingOptimizeMaxDisplacementLower;
}

inline void rk::dnspack::ReOptimizeVariables::setDensestPackingOptimizeDecreaseFactor(const double& densestPackingOptimizeDecreaseFactor)
{
	if (densestPackingOptimizeDecreaseFactor < 0.0)
	{
		throw std::logic_error("densestPackingOptimizeDecreaseFactor is less than zero.");
	}

	m_densestPackingOptimizeDecreaseFactor = densestPackingOptimizeDecreaseFactor;
}

inline void rk::dnspack::ReOptimizeVariables::setDensestPackingOptimizeIterationNumber(const int& densestPackingOptimizeIteratorNumber)
{
	if (densestPackingOptimizeIteratorNumber < 0)
	{
		throw std::logic_error("densestPackingOptimizeIteratorNumber is less than zero.");
	}

	m_densestPackingOptimizeIterationNumber = densestPackingOptimizeIteratorNumber;
}

inline void rk::dnspack::ReOptimizeVariables::setDensestPackingOptimizeNeighborUpdateInterval(const int& densestPackingOptimizeNeighborUpdateInterval)
{
	if (densestPackingOptimizeNeighborUpdateInterval <= 0)
	{
		throw std::logic_error("densestPackingOptimizeNeighborUpdateInterval have to be larger than zero.");
	}

	m_densestPackingOptimizeNeighborUpdateInterval = densestPackingOptimizeNeighborUpdateInterval;
}

	// ReOptimization Parameters
// ********************************************************************************************************************************************************************

// ********************************************************************************************************************************************************************
	// Conditions

inline int rk::dnspack::ReOptimizeVariables::getReOptimizeStructureNumber() const noexcept
{
	return m_reOptimizeStructureNumber;
}

inline int rk::dnspack::ReOptimizeVariables::getReOptimizeRepetitionNumber() const noexcept
{
	return m_reOptimizeRepetitionNumber;
}

inline double rk::dnspack::ReOptimizeVariables::getOutputMinimumZ() const noexcept
{
	return m_outputMinimumZ;
}

inline void rk::dnspack::ReOptimizeVariables::setReOptimizeStructureNumber(const int& reOptimizeStructureNumber)
{
	if (reOptimizeStructureNumber < 1)
	{
		throw std::logic_error("thresholdTrialNumber is less than one.");
	}

	m_reOptimizeStructureNumber = reOptimizeStructureNumber;
}

inline void rk::dnspack::ReOptimizeVariables::setReOptimizeRepetitionNumber(const int& reOptimizeRepetitionNumber)
{
	if (reOptimizeRepetitionNumber < 1)
	{
		throw std::logic_error("maxTrialNumber is less than one.");
	}

	m_reOptimizeRepetitionNumber = reOptimizeRepetitionNumber;
}

inline void rk::dnspack::ReOptimizeVariables::setOutputMinimumZ(const double& outputMinimumZ)
{
	if (0.0 <= outputMinimumZ)
	{
		throw std::logic_error("outputMinimumZ is more than or equal to zero.");
	}

	m_outputMinimumZ = outputMinimumZ;
}

	// Conditions
// ********************************************************************************************************************************************************************


#endif // !RK_DNSPACK_REOPTIMIZEVARIABLES_H

