#ifndef RK_DNSPACK_READINPUTFILE_H
#define RK_DNSPACK_READINPUTFILE_H

#include <string>
#include <vector>

#include "BasisComposition.h"

#include "ExploreVariables.h"
#include "ReOptimizeVariables.h"


namespace rk
{
	namespace dnspack
	{
		class ReadInputFile
		{
		public:
			explicit ReadInputFile(const std::string& fileName);
			virtual ~ReadInputFile();

		public:
			void initialize();

			int getThreadNumber() const noexcept;
			const std::string& getOutputDirectoryName() const noexcept;
			const std::vector<BasisComposition>& getTrialBasisCompositions() const noexcept;
			const ExploreVariables& getExploreVariables() const noexcept;
			const ReOptimizeVariables& getReOptimizeVariables() const noexcept;

		private:
			const std::string m_fileName;

			int m_threadNumber;
			std::string m_outputDirectoryName;
			std::vector<BasisComposition> m_trialBasisCompositions;
			ExploreVariables m_exploreVariables;
			ReOptimizeVariables m_reOptimizeVariables;

		private:
			ReadInputFile(const ReadInputFile&) = delete;
			ReadInputFile(ReadInputFile&&) noexcept = delete;
			ReadInputFile& operator=(const ReadInputFile&) = delete;
			ReadInputFile& operator=(ReadInputFile&&) noexcept = delete;
		};
	}
}


inline int rk::dnspack::ReadInputFile::getThreadNumber() const noexcept
{
	return m_threadNumber;
}

inline const std::string& rk::dnspack::ReadInputFile::getOutputDirectoryName() const noexcept
{
	return m_outputDirectoryName;
}

inline const std::vector<rk::dnspack::BasisComposition>& rk::dnspack::ReadInputFile::getTrialBasisCompositions() const noexcept
{
	return m_trialBasisCompositions;
}

inline const rk::dnspack::ExploreVariables& rk::dnspack::ReadInputFile::getExploreVariables() const noexcept
{
	return m_exploreVariables;
}

inline const rk::dnspack::ReOptimizeVariables& rk::dnspack::ReadInputFile::getReOptimizeVariables() const noexcept
{
	return m_reOptimizeVariables;
}


#endif // !RK_DNSPACK_READINPUTFILE_H
