#ifndef RK_DNSPACK_SPHERE_H
#define RK_DNSPACK_SPHERE_H

#include <stdexcept>

#include "FractionalCoordinate.h"

namespace rk
{
	namespace dnspack
	{
		class Sphere
		{
		public:
			Sphere() noexcept;
			explicit Sphere(const double radius);
			virtual ~Sphere();

		public:
			Sphere(const Sphere& other) noexcept;
			Sphere(Sphere&& other) noexcept;
			Sphere& operator=(const Sphere& other) noexcept;
			Sphere& operator=(Sphere&& other) noexcept;

			bool operator==(const Sphere& other) const noexcept;
			bool operator!=(const Sphere& other) const noexcept;

		public:
			double getRadius() const noexcept;
			void setRadius(const double radius);

			const FractionalCoordinate& getFractionalCoordinate() const noexcept;
			FractionalCoordinate& getFractionalCoordinate() noexcept;

			void refineFractionalCoordinate() noexcept;

		private:
			double m_radius;
			FractionalCoordinate m_fractionalCoordinate;
		};
	}
}

inline rk::dnspack::Sphere& rk::dnspack::Sphere::operator=(const Sphere& other) noexcept
{
	m_radius = other.m_radius;
	m_fractionalCoordinate = other.m_fractionalCoordinate;

	return *this;
}

inline rk::dnspack::Sphere& rk::dnspack::Sphere::operator=(Sphere&& other) noexcept
{
	m_radius = other.m_radius;
	m_fractionalCoordinate = std::move(other.m_fractionalCoordinate);

	return *this;
}

inline bool rk::dnspack::Sphere::operator==(const Sphere& other) const noexcept
{
	return (m_radius == other.m_radius && m_fractionalCoordinate == other.m_fractionalCoordinate);
}

inline bool rk::dnspack::Sphere::operator!=(const Sphere& other) const noexcept
{
	return !(*this == other);
}

inline double rk::dnspack::Sphere::getRadius() const noexcept
{
	return m_radius;
}

inline void rk::dnspack::Sphere::setRadius(const double radius)
{
	if (radius <= 0)
	{
		throw std::invalid_argument("radius should be larger than 0.");
	}

	m_radius = radius;
}

inline const rk::dnspack::FractionalCoordinate& rk::dnspack::Sphere::getFractionalCoordinate() const noexcept
{
	return m_fractionalCoordinate;
}

inline rk::dnspack::FractionalCoordinate& rk::dnspack::Sphere::getFractionalCoordinate() noexcept
{
	return m_fractionalCoordinate;
}

inline void rk::dnspack::Sphere::refineFractionalCoordinate() noexcept
{
	for (; m_fractionalCoordinate.a < 0.0; )
	{
		m_fractionalCoordinate.a += 1.0;
	}

	for (; 1.0 < m_fractionalCoordinate.a; )
	{
		m_fractionalCoordinate.a -= 1.0;
	}

	for (; m_fractionalCoordinate.b < 0.0; )
	{
		m_fractionalCoordinate.b += 1.0;
	}

	for (; 1.0 < m_fractionalCoordinate.b; )
	{
		m_fractionalCoordinate.b -= 1.0;
	}

	for (; m_fractionalCoordinate.c < 0.0; )
	{
		m_fractionalCoordinate.c += 1.0;
	}

	for (; 1.0 < m_fractionalCoordinate.c; )
	{
		m_fractionalCoordinate.c -= 1.0;
	}
}


#endif // !RK_DNSPACK_SPHERE_H
