#include "Clock.h"

#include <ctime>
#include <stdexcept>

using namespace rk::dnspack;


std::map<std::string, std::chrono::time_point<std::chrono::steady_clock>> Clock::m_stopwatches;


Clock::Clock()
{
}

Clock::~Clock()
{
}

bool Clock::stopwatchBegin(const std::string& key)
{
	std::map<std::string, std::chrono::time_point<std::chrono::steady_clock>>::iterator iter = m_stopwatches.find(key);
	std::map<std::string, std::chrono::time_point<std::chrono::steady_clock>>::iterator iterEnd = m_stopwatches.end();

	if (iter == iterEnd)
	{
		std::chrono::time_point<std::chrono::steady_clock> nowTime = std::chrono::steady_clock::now();

		m_stopwatches.emplace(key, nowTime);
		return true;
	}

	else
	{
		return false;
	}
}

bool Clock::stopwatchFinish(const std::string& key)
{
	std::map<std::string, std::chrono::time_point<std::chrono::steady_clock>>::iterator iter = m_stopwatches.find(key);
	std::map<std::string, std::chrono::time_point<std::chrono::steady_clock>>::iterator iterEnd = m_stopwatches.end();

	if (iter == iterEnd)
	{
		return false;
	}

	else
	{
		m_stopwatches.erase(iter);
		return true;
	}
}

std::string Clock::getElapsedTime(const std::string& key)
{
	std::map<std::string, std::chrono::time_point<std::chrono::steady_clock>>::iterator iter = m_stopwatches.find(key);
	std::map<std::string, std::chrono::time_point<std::chrono::steady_clock>>::iterator iterEnd = m_stopwatches.end();

	if (iter == iterEnd)
	{
		throw std::logic_error("Clock::getElapsedTime(const std::string& key) -> m_stopwatches does not contain the key.");
	}

	else
	{
		std::chrono::time_point<std::chrono::steady_clock> nowTime = std::chrono::steady_clock::now();

		std::chrono::seconds begin = std::chrono::duration_cast<std::chrono::seconds>(iter->second.time_since_epoch());
		std::chrono::seconds end = std::chrono::duration_cast<std::chrono::seconds>(nowTime.time_since_epoch());

		std::chrono::seconds s = end - begin;

		long long time = s.count();

		long long hour = time / 3600;
		long long minute = (time - 3600 * hour) / 60;
		long long second = time - 3600 * hour - 60 * minute;


		std::string timeString;
		{
			timeString += std::to_string(hour) + " h ";
			timeString += std::to_string(minute) + " min ";
			timeString += std::to_string(second) + " sec";
		}

		return timeString;
	}
}


#ifdef _MSC_VER

std::string Clock::getNowTime()
{
	char buf[26];
	errno_t err;

	std::chrono::system_clock::time_point timePoint = std::chrono::system_clock::now();
	std::time_t time = std::chrono::system_clock::to_time_t(timePoint);
	err = ctime_s(buf, 26, &time);

	if (err == 0)
	{
		std::string timeString = buf;
		return timeString;
	}

	else
	{
		throw std::runtime_error("Could not get the current time");
	}
}


#else

std::string Clock::getNowTime()
{
	std::chrono::system_clock::time_point timePoint = std::chrono::system_clock::now();
	std::time_t time = std::chrono::system_clock::to_time_t(timePoint);

	std::string timeString = ctime(&time);
	return timeString;
}


#endif // _MSC_VER
