#include "MakeDirectory.h"

#include <stdexcept>

using namespace rk::dnspack;


#ifdef _MSC_VER

MakeDirectory::MakeDirectory(const std::string& parentDirectoryPath)
	: m_parentDirectoryPath(parentDirectoryPath)
{
	if (!(std::filesystem::is_directory(m_parentDirectoryPath)))
	{
		std::string message;
		message += parentDirectoryPath;
		message += " is not a directory.";

		throw std::invalid_argument(message);
	}
}

MakeDirectory::~MakeDirectory()
{
}

bool MakeDirectory::operator()(const std::string& childDirectoryName)
{
	if (!(childDirectoryName.back() == '/'))
	{
		throw std::invalid_argument("The last literature of childDirectoryName is not /");
	}

	std::filesystem::path childDirectoryPath = m_parentDirectoryPath;
	childDirectoryPath += std::filesystem::path(childDirectoryName);

	if (std::filesystem::is_directory(childDirectoryPath))
	{
		return false;
	}

	else
	{
		if (!(std::filesystem::create_directory(childDirectoryPath)))
		{
			std::string message;
			message += "couldn't make the directory of ";
			message += childDirectoryPath.string();

			throw std::invalid_argument(message);
		}
	}

	return true;
}


#else

#include <sys/stat.h>

MakeDirectory::MakeDirectory(const std::string& parentDirectoryPath)
	: m_parentDirectoryPath{ parentDirectoryPath }
{
	if (!(m_parentDirectoryPath.back() == '/'))
	{
		throw std::invalid_argument("The last literature of parentDirectoryPath is not /");
	}

	struct stat s;
	stat(m_parentDirectoryPath.c_str(), &s);

	if (!S_ISDIR(s.st_mode))
	{
		std::string message;
		message += m_parentDirectoryPath;
		message += " is not a directory.";

		throw std::invalid_argument(message);
	}
}

MakeDirectory::~MakeDirectory()
{
}

bool MakeDirectory::operator()(const std::string& childDirectoryName)
{
	if (!(childDirectoryName.back() == '/'))
	{
		throw std::invalid_argument("The last literature of childDirectoryName is not /");
	}

	std::string childDirectoryPathName = m_parentDirectoryPath + childDirectoryName;
	mkdir(childDirectoryPathName.c_str(), 0775);

	struct stat s;
	stat(childDirectoryPathName.c_str(), &s);

	if (!S_ISDIR(s.st_mode))
	{
		std::string message;
		message += "couldn't make the directory of ";
		message += childDirectoryPathName;

		throw std::invalid_argument(message);
	}

	return true;
}


#endif // _MSC_VER
