#include "RandomFluctuate.h"

#include <random>

using namespace rk::dnspack;


RandomFluctuate::RandomFluctuate(const double maxFluctuation)
	: m_maxFluctuation(maxFluctuation)
{
}

RandomFluctuate::~RandomFluctuate()
{
}

void RandomFluctuate::operator()(PackingStructure& packingStructure)
{
	std::random_device seed_gen;
	std::default_random_engine engine(seed_gen());

	std::uniform_real_distribution<double> displacementDist((-1.0 * m_maxFluctuation), m_maxFluctuation);


	const BravaisLattice& bravaisLattice(packingStructure.getBravaisLattice());

	CartesianCoordinate reciprocalLatticeVector[3];
	{
		double volume = bravaisLattice.getVolume();

		reciprocalLatticeVector[0] = crossProduct(bravaisLattice[1], bravaisLattice[2]);
		reciprocalLatticeVector[0].scaling((1 / volume));

		reciprocalLatticeVector[1] = crossProduct(bravaisLattice[2], bravaisLattice[0]);
		reciprocalLatticeVector[1].scaling((1 / volume));

		reciprocalLatticeVector[2] = crossProduct(bravaisLattice[0], bravaisLattice[1]);
		reciprocalLatticeVector[2].scaling((1 / volume));
	}

	for (auto& sphere : packingStructure.getBasis())
	{
		CartesianCoordinate positionVector;
		{
			positionVector += scaling(sphere.getFractionalCoordinate().a, bravaisLattice[0]);
			positionVector += scaling(sphere.getFractionalCoordinate().b, bravaisLattice[1]);
			positionVector += scaling(sphere.getFractionalCoordinate().c, bravaisLattice[2]);
		}

		positionVector.x += displacementDist(engine);
		positionVector.y += displacementDist(engine);
		positionVector.z += displacementDist(engine);

		sphere.getFractionalCoordinate().a = innerProduct(reciprocalLatticeVector[0], positionVector);
		sphere.getFractionalCoordinate().b = innerProduct(reciprocalLatticeVector[1], positionVector);
		sphere.getFractionalCoordinate().c = innerProduct(reciprocalLatticeVector[2], positionVector);

		sphere.refineFractionalCoordinate();
	}
}
