#include "ReadDatFile.h"

#include <fstream>
#include <stdexcept>
#include <sstream>
#include <regex>
#include <vector>
#include <set>

using namespace rk::dnspack;

ReadDatFile::ReadDatFile()
{
}

ReadDatFile::~ReadDatFile()
{
}

PackingStructure ReadDatFile::operator()(const std::string& fileName)
{
// ******************************************************************************************************************************************
	// Read input file

	std::string fileString;
	{
		std::ifstream inputFileStream;
		inputFileStream.open(fileName, std::ios_base::in);

		if (!inputFileStream)
		{
			std::string message{ "could not open " };
			message += fileName + '.';

			throw std::runtime_error(message);
		}

		for (std::string stringLine; std::getline(inputFileStream, stringLine);)
		{
			fileString += stringLine + "\n";
		}

		inputFileStream.close();
	}

	// Read input file
// ******************************************************************************************************************************************

// ******************************************************************************************************************************************
	// Read composition

	std::map<std::string, double> nameAndRadii;
	{
		std::size_t initialPos = fileString.find("<Definition.of.Sphere.Species");
		std::size_t endPos = fileString.find("Definition.of.Sphere.Species>");

		if (endPos <= initialPos)
		{
			throw std::runtime_error("couldn't get the Definition.of.Sphere.Species.");
		}

		else
		{
			if (initialPos != std::string::npos && endPos != std::string::npos)
			{
				std::set<double> radii;

				std::size_t tempPos = fileString.find_first_of("\n", initialPos);
				std::string sphereNameString{ fileString, (tempPos + 1), (endPos - tempPos - 1) };
				std::stringstream sphereNameStream{ sphereNameString };

				std::regex pat("[\\s]*([\\w]+)[\\s]+([\\d]+[\\.]?[\\d]*)[\\s]*");

				for (std::string line; std::getline(sphereNameStream, line);)
				{
					std::smatch compositionSm;

					if (std::regex_match(line, compositionSm, pat))
					{
						nameAndRadii.emplace(compositionSm.str(1), std::stod(compositionSm.str(2)));
						radii.emplace(std::stod(compositionSm.str(2)));
					}

					else
					{
						throw std::runtime_error("couldn't get the Definition.of.Sphere.Species.");
					}
				}

				double maxRadius = *(std::prev(radii.end()));
				double difference = 1.0 - maxRadius;

				if (0.0000000000001 < std::abs(difference))
				{
					throw std::runtime_error("The maximum radius is not 1.0.");
				}
			}

			else
			{
				throw std::runtime_error("couldn't get the Definition.of.Sphere.Species.");
			}
		}
	}

	// Read composition
// ******************************************************************************************************************************************


// ******************************************************************************************************************************************
	// Read structure

	{
		std::regex pat("[\\s]*Atoms.SpeciesAndCoordinates.Unit[\\s]+([\\w]+)[\\s]*");
		std::smatch numSm;

		if (std::regex_search(fileString, numSm, pat))
		{
			if (numSm.str(1) != "FRAC")
			{
				throw std::runtime_error("Coordinates have to be written in fractional coordinates.");
			}
		}

		else
		{
			throw std::runtime_error("couldn't get Atoms.SpeciesAndCoordinates.Unit.");
		}
	}
	
	std::vector<Sphere> basis;
	{
		std::size_t initialPos = fileString.find("<Atoms.SpeciesAndCoordinates");
		std::size_t endPos = fileString.find("Atoms.SpeciesAndCoordinates>");

		if (initialPos != std::string::npos && endPos != std::string::npos)
		{
			std::size_t tempPos = fileString.find_first_of("\n", initialPos);
			std::string coordinateString{ fileString, (tempPos + 1), (endPos - tempPos - 1) };
			std::stringstream coordinateStream{ coordinateString };

			std::regex pat("[\\s]*[\\d]+[\\s]+([\\w]+)[\\s]+([\\-]?[\\d]+[\\.]?[\\d]*)[\\s]+([\\-]?[\\d]+[\\.]?[\\d]*)[\\s]+([\\-]?[\\d]+[\\.]?[\\d]*)[\\s]*");

			for (std::string line; std::getline(coordinateStream, line);)
			{
				std::smatch coordinateSm;

				if (std::regex_match(line, coordinateSm, pat))
				{
					Sphere sphere;
					sphere.setRadius(nameAndRadii.at(coordinateSm.str(1)));

					sphere.getFractionalCoordinate().a = std::stod(coordinateSm.str(2));
					sphere.getFractionalCoordinate().b = std::stod(coordinateSm.str(3));
					sphere.getFractionalCoordinate().c = std::stod(coordinateSm.str(4));

					basis.push_back(std::move(sphere));
				}

				else
				{
					throw std::runtime_error("couldn't get the Atoms.SpeciesAndCoordinates.");
				}
			}
		}

		else
		{
			throw std::runtime_error("couldn't get the Atoms.SpeciesAndCoordinates.");
		}
	}

	BravaisLattice bravaisLattice;
	{
		std::size_t initialPos = fileString.find("<Atoms.UnitVectors");
		std::size_t endPos = fileString.find("Atoms.UnitVectors>");

		if (initialPos != std::string::npos && endPos != std::string::npos)
		{
			std::size_t tempPos = fileString.find_first_of("\n", initialPos);
			std::string latticeString{ fileString, (tempPos + 1), (endPos - tempPos - 1) };
			std::stringstream latticeStream{ latticeString };

			std::regex pat("[\\s]*([\\-]?[\\d]+[\\.]?[\\d]*)[\\s]+([\\-]?[\\d]+[\\.]?[\\d]*)[\\s]+([\\-]?[\\d]+[\\.]?[\\d]*)[\\s]*");

			int label = 0;

			for (std::string line; std::getline(latticeStream, line);)
			{
				std::smatch latticeVectorSm;

				if (std::regex_match(line, latticeVectorSm, pat))
				{
					bravaisLattice[label].x = std::stod(latticeVectorSm.str(1));
					bravaisLattice[label].y = std::stod(latticeVectorSm.str(2));
					bravaisLattice[label].z = std::stod(latticeVectorSm.str(3));

					++label;
				}

				else
				{
					throw std::runtime_error("couldn't get the Atoms.UnitVectors.");
				}
			}
		}

		else
		{
			throw std::runtime_error("couldn't get the Atoms.UnitVectors.");
		}
	}

	// Read structure
// ******************************************************************************************************************************************

	return PackingStructure(bravaisLattice, std::move(basis));
}
