#include "RefineUnitCell.h"

#include <vector>

using namespace rk::dnspack;


RefineUnitCell::RefineUnitCell()
{
}

RefineUnitCell::~RefineUnitCell()
{
}

void RefineUnitCell::operator()(PackingStructure& packingStructure)
{
	const BravaisLattice initialBravaisLattice = packingStructure.getBravaisLattice();
	BravaisLattice bestBravaisLattice = initialBravaisLattice;

// ********************************************************************************************************************************************************************
	// At the beginning, we refine the initialBravaisLattice[1]

	{
		CartesianCoordinate bLatticeCandidate[5];
		{
			bLatticeCandidate[0] = bestBravaisLattice[1] - bestBravaisLattice[0] - bestBravaisLattice[0];
			bLatticeCandidate[1] = bestBravaisLattice[1] - bestBravaisLattice[0];
			bLatticeCandidate[2] = bestBravaisLattice[1];
			bLatticeCandidate[3] = bestBravaisLattice[1] + bestBravaisLattice[0];
			bLatticeCandidate[4] = bestBravaisLattice[1] + bestBravaisLattice[0] + bestBravaisLattice[0];
		}

		std::size_t bMinimumLabel = 0;
		double bMinimumSizeSquare = 10000.0;

		for (std::size_t label = 0; label < 5; ++label)
		{
			double sizeSquare = bLatticeCandidate[label].getNormSquare();

			if (sizeSquare < bMinimumSizeSquare)
			{
				bMinimumSizeSquare = sizeSquare;
				bMinimumLabel = label;
			}
		}

		bestBravaisLattice[1] = bLatticeCandidate[bMinimumLabel];
	}

	// At the beginning, we refine the initialBravaisLattice[1]
// ********************************************************************************************************************************************************************

// ********************************************************************************************************************************************************************
	// We refine the initialBravaisLattice[0] and initialBravaisLattice[1] while the refinement occurs.

	while (true)
	{
		{
			CartesianCoordinate aLatticeCandidate[5];
			{
				aLatticeCandidate[0] = bestBravaisLattice[0] - bestBravaisLattice[1] - bestBravaisLattice[1];
				aLatticeCandidate[1] = bestBravaisLattice[0] - bestBravaisLattice[1];
				aLatticeCandidate[2] = bestBravaisLattice[0];
				aLatticeCandidate[3] = bestBravaisLattice[0] + bestBravaisLattice[1];
				aLatticeCandidate[4] = bestBravaisLattice[0] + bestBravaisLattice[1] + bestBravaisLattice[1];
			}

			std::size_t aMinimumLabel = 0;
			double aMinimumSizeSquare = 10000.0;

			for (std::size_t label = 0; label < 5; ++label)
			{
				double sizeSquare = aLatticeCandidate[label].getNormSquare();

				if (sizeSquare < aMinimumSizeSquare)
				{
					aMinimumSizeSquare = sizeSquare;
					aMinimumLabel = label;
				}
			}

			if (aMinimumLabel == 2)
			{
				break;
			}

			else
			{
				bestBravaisLattice[0] = aLatticeCandidate[aMinimumLabel];
			}
		}

		{
			CartesianCoordinate bLatticeCandidate[5];
			{
				bLatticeCandidate[0] = bestBravaisLattice[1] - bestBravaisLattice[0] - bestBravaisLattice[0];
				bLatticeCandidate[1] = bestBravaisLattice[1] - bestBravaisLattice[0];
				bLatticeCandidate[2] = bestBravaisLattice[1];
				bLatticeCandidate[3] = bestBravaisLattice[1] + bestBravaisLattice[0];
				bLatticeCandidate[4] = bestBravaisLattice[1] + bestBravaisLattice[0] + bestBravaisLattice[0];
			}

			std::size_t bMinimumLabel = 0;
			double bMinimumSizeSquare = 10000.0;

			for (std::size_t label = 0; label < 5; ++label)
			{
				double sizeSquare = bLatticeCandidate[label].getNormSquare();

				if (sizeSquare < bMinimumSizeSquare)
				{
					bMinimumSizeSquare = sizeSquare;
					bMinimumLabel = label;
				}
			}

			if (bMinimumLabel == 2)
			{
				break;
			}

			else
			{
				bestBravaisLattice[1] = bLatticeCandidate[bMinimumLabel];
			}
		}
	}

	// We refine the initialBravaisLattice[0] and initialBravaisLattice[1] while the refinement occurs.
// ********************************************************************************************************************************************************************

// ********************************************************************************************************************************************************************
	// Finally, we refine the initialBravaisLattice[2]

	{
		std::size_t cMinimumLabel = 0;
		double cMinimumSizeSquare = 10000.0;

		CartesianCoordinate criteriaVector = crossProduct(bestBravaisLattice[0], bestBravaisLattice[1]);

		for (int Qx = -2; Qx < 3; ++Qx)
		{
			for (int Qy = -2; Qy < 3; ++Qy)
			{
				CartesianCoordinate cLatticeCandidate = initialBravaisLattice[2];
				cLatticeCandidate += scaling(static_cast<double>(Qx), bestBravaisLattice[0]);
				cLatticeCandidate += scaling(static_cast<double>(Qy), bestBravaisLattice[1]);

				double sizeSquare = cLatticeCandidate.getNormSquare();

				if (sizeSquare < cMinimumSizeSquare)
				{
					cMinimumSizeSquare = sizeSquare;
					bestBravaisLattice[2] = cLatticeCandidate;
				}
			}
		}
	}

	// Finally, we refine the initialBravaisLattice[2].
// ********************************************************************************************************************************************************************

// ********************************************************************************************************************************************************************
	// correlate the fractional coordinates

	CartesianCoordinate reciprocalLatticeVector[3];
	{
		double volume = bestBravaisLattice.getVolume();

		reciprocalLatticeVector[0] = crossProduct(bestBravaisLattice[1], bestBravaisLattice[2]);
		reciprocalLatticeVector[0].scaling((1.0 / volume));

		reciprocalLatticeVector[1] = crossProduct(bestBravaisLattice[2], bestBravaisLattice[0]);
		reciprocalLatticeVector[1].scaling((1.0 / volume));

		reciprocalLatticeVector[2] = crossProduct(bestBravaisLattice[0], bestBravaisLattice[1]);
		reciprocalLatticeVector[2].scaling((1.0 / volume));
	}

	std::vector<Sphere>& basis = packingStructure.getBasis();

	for (auto& sphere : basis)
	{
		CartesianCoordinate position;
		{
			position += scaling(sphere.getFractionalCoordinate().a, initialBravaisLattice[0]);
			position += scaling(sphere.getFractionalCoordinate().b, initialBravaisLattice[1]);
			position += scaling(sphere.getFractionalCoordinate().c, initialBravaisLattice[2]);
		}

		sphere.getFractionalCoordinate().a = innerProduct(position, reciprocalLatticeVector[0]);
		sphere.getFractionalCoordinate().b = innerProduct(position, reciprocalLatticeVector[1]);
		sphere.getFractionalCoordinate().c = innerProduct(position, reciprocalLatticeVector[2]);
		sphere.refineFractionalCoordinate();
	}

	// correlate the fractional coordinates
// ********************************************************************************************************************************************************************

	packingStructure.getBravaisLattice() = bestBravaisLattice;
}
